@@ -39,6 +39,10 @@ mod imp {
         detect_codes: Cell<bool>,
         #[property(get, set = Self::set_camera, nullable, explicit_notify)]
         camera: RefCell<Option<crate::Camera>>,
+        #[property(get = Self::is_recording, name = "is-recording", type = bool)]
+        pub is_recording_video: RefCell<Option<PathBuf>>,
+        #[property(get, set = Self::set_disable_audio_recording, explicit_notify)]
+        disable_audio_recording: Cell<bool>,
 
         pub zbar_branch: RefCell<Option<gst::Element>>,
         pub devices: OnceCell<crate::DeviceProvider>,
@@ -50,8 +54,6 @@ mod imp {
         pub tee: OnceCell<crate::PipelineTee>,
         pub bus_watch: OnceCell<gst::bus::BusWatchGuard>,
 
-        // TODO Port to gio::Task,
-        pub is_recording_video: RefCell<Option<PathBuf>>,
         pub is_stopping_recording: Cell<bool>,
         pub is_taking_picture: Cell<bool>,
         pub is_front_camera: Cell<bool>,
@@ -59,6 +61,7 @@ mod imp {
         pub timeout_handler: RefCell<Option<glib::SourceId>>,
 
         pub picture: gtk::Picture,
+        pub offload: gtk::GraphicsOffload,
     }
 
     impl Viewfinder {
@@ -72,6 +75,10 @@ mod imp {
             }
         }
 
+        fn is_recording(&self) -> bool {
+            self.is_recording_video.borrow().is_some()
+        }
+
         fn detect_codes(&self) -> bool {
             self.zbar_branch.borrow().is_some()
         }
@@ -160,6 +167,27 @@ mod imp {
 
             obj.notify_camera();
         }
+
+        fn set_disable_audio_recording(&self, value: bool) {
+            let obj = self.obj();
+
+            if value == self.disable_audio_recording.replace(value) {
+                return;
+            }
+
+            if matches!(
+                self.camerabin().current_state(),
+                gst::State::Playing | gst::State::Paused
+            ) {
+                obj.stop_stream();
+                obj.setup_recording();
+                obj.start_stream();
+            } else {
+                obj.setup_recording();
+            }
+
+            obj.notify_disable_audio_recording();
+        }
     }
 
     #[glib::object_subclass]
@@ -189,20 +217,25 @@ mod imp {
             self.camerabin.set(camerabin.clone()).unwrap();
 
             let bus = self.camerabin().bus().unwrap();
-            let watch = bus.add_watch_local(
-                glib::clone!(@weak obj => @default-return glib::ControlFlow::Break, move |_, msg| {
-                    obj.on_bus_message(msg);
-                    glib::ControlFlow::Continue
-                }),
-            )
-            .unwrap();
+            let watch = bus
+                .add_watch_local(glib::clone!(
+                    #[weak]
+                    obj,
+                    #[upgrade_or]
+                    glib::ControlFlow::Break,
+                    move |_, msg| {
+                        obj.on_bus_message(msg);
+                        glib::ControlFlow::Continue
+                    }
+                ))
+                .unwrap();
             self.bus_watch.set(watch).unwrap();
 
             let tee = crate::PipelineTee::new();
 
             let paintablesink = gst::ElementFactory::make("gtk4paintablesink")
                 .build()
-                .unwrap();
+                .expect("Missing gst-plugin-gtk4");
 
             let paintable = paintablesink.property::<gdk::Paintable>("paintable");
             let is_gl_supported = paintable
@@ -234,16 +267,27 @@ mod imp {
             tee.add_branch(&sink);
             camerabin.set_property("viewfinder-sink", &tee);
 
+            let videoconvert_video = gst::ElementFactory::make("videoconvert")
+                .build()
+                .expect("Missing GStreamer Base Plug-ins");
+            camerabin.set_property("video-filter", &videoconvert_video);
+
+            let videoconvert_image = gst::ElementFactory::make("videoconvert")
+                .build()
+                .expect("Missing GStreamer Base Plug-ins");
+            camerabin.set_property("image-filter", &videoconvert_image);
+
             self.sink_paintable.set(paintablesink).unwrap();
 
             self.picture
                 .set_accessible_role(gtk::AccessibleRole::Presentation);
             self.picture.set_hexpand(true);
             self.picture.set_vexpand(true);
-            self.picture.set_content_fit(gtk::ContentFit::Fill);
-            self.picture.set_parent(&*obj);
             self.picture.set_paintable(Some(&paintable));
 
+            self.offload.set_child(Some(&self.picture));
+            self.offload.set_parent(&*obj);
+
             self.tee.set(tee).unwrap();
 
             let devices = crate::DeviceProvider::instance();
@@ -253,31 +297,48 @@ mod imp {
             if devices.started() {
                 obj.init();
             } else {
-                devices.connect_started_notify(glib::clone!(@weak obj => move |_| {
-                    obj.init();
-                }));
+                devices.connect_started_notify(glib::clone!(
+                    #[weak]
+                    obj,
+                    move |_| {
+                        obj.init();
+                    }
+                ));
             }
 
-            devices.connect_camera_added(glib::clone!(@weak obj => move |_, camera| {
-                if matches!(obj.state(), ViewfinderState::NoCameras | ViewfinderState::Loading | ViewfinderState::Error) {
-                    obj.imp().set_state(ViewfinderState::Ready);
-                    obj.set_camera(Some(camera.clone()));
-                }
-            }));
-
-            devices.connect_camera_removed(glib::clone!(@weak obj => move |devices, camera| {
-                let imp = obj.imp();
-                if Some(camera) == imp.camera.borrow().as_ref() {
-                    obj.cancel_current_operation();
-
-                    let next_camera = devices.camera(0);
-                    let is_none = next_camera.is_none();
-                    obj.set_camera(next_camera);
-                    if is_none {
-                        obj.imp().set_state(ViewfinderState::NoCameras);
+            devices.connect_camera_added(glib::clone!(
+                #[weak]
+                obj,
+                move |_, camera| {
+                    if matches!(
+                        obj.state(),
+                        ViewfinderState::NoCameras
+                            | ViewfinderState::Loading
+                            | ViewfinderState::Error
+                    ) {
+                        obj.imp().set_state(ViewfinderState::Ready);
+                        obj.set_camera(Some(camera.clone()));
                     }
                 }
-            }));
+            ));
+
+            devices.connect_camera_removed(glib::clone!(
+                #[weak]
+                obj,
+                move |devices, camera| {
+                    let imp = obj.imp();
+                    if Some(camera) == imp.camera.borrow().as_ref() {
+                        obj.cancel_current_operation();
+
+                        let next_camera = devices.camera(0);
+                        let is_none = next_camera.is_none();
+                        obj.set_camera(next_camera);
+                        if is_none {
+                            obj.imp().set_state(ViewfinderState::NoCameras);
+                        }
+                    }
+                }
+            ));
 
             log::debug!("Setup recording");
             obj.setup_recording();
@@ -293,7 +354,7 @@ mod imp {
                 log::error!("Could not stop camerabin: {err}");
             }
 
-            self.picture.unparent();
+            self.offload.unparent();
         }
 
         fn signals() -> &'static [glib::subclass::Signal] {
@@ -333,20 +394,16 @@ mod imp {
 
         fn snapshot(&self, snapshot: &gtk::Snapshot) {
             let w = self.obj().width() as f32;
+            let h = self.obj().height() as f32;
+
+            // Make the background black
+            let rect = graphene::Rect::new(0.0, 0.0, w, h);
+            snapshot.append_color(&gdk::RGBA::BLACK, &rect);
 
-            // This is the composition of translate (-w / 2.0, 0.0), map x to
-            // -x, and translate (w / 2.0 , 0.0). Note that gsk matrices are
-            // transposed (they act on row vectors).
             if self.is_front_camera.get() {
-                #[rustfmt::skip]
-                let flip_matrix = graphene::Matrix::from_float([
-                    -1.0,  0.0,  0.0,  0.0,
-                     0.0,  1.0,  0.0,  0.0,
-                     0.0,  0.0,  1.0,  0.0,
-                       w,  0.0,  0.0,  1.0,
-                ]);
                 snapshot.save();
-                snapshot.transform_matrix(&flip_matrix);
+                snapshot.translate(&graphene::Point::new(w, 0.0));
+                snapshot.scale(-1.0, 1.0);
                 self.parent_snapshot(snapshot);
                 snapshot.restore();
             } else {
@@ -521,8 +578,13 @@ impl Viewfinder {
         }
 
         // Set after we cannot fail anymore.
-        imp.is_recording_video
-            .replace(Some(location.as_ref().to_owned()));
+        if !imp
+            .is_recording_video
+            .replace(Some(location.as_ref().to_owned()))
+            .is_some_and(|old| old == location.as_ref())
+        {
+            self.notify_is_recording();
+        };
 
         let camerabin = imp.camerabin();
         camerabin.set_property_from_str("mode", "mode-video");
@@ -562,15 +624,6 @@ impl Viewfinder {
         Ok(())
     }
 
-    /// Gets whether a recording is in progress.
-    ///
-    /// # Returns
-    ///
-    /// if a recording is in progress.
-    pub fn is_recording(&self) -> bool {
-        self.imp().is_recording_video.borrow().is_some()
-    }
-
     pub fn connect_picture_done<F: Fn(&Self, Option<&gio::File>) + 'static>(&self, f: F) {
         self.connect_closure(
             "picture-done",
@@ -603,9 +656,13 @@ impl Viewfinder {
 
     /// Starts the viewfinder.
     pub fn start_stream(&self) {
-        glib::spawn_future_local(glib::clone!(@weak self as obj => async move {
-            obj.change_state_inner(gst::State::Playing).await;
-        }));
+        glib::spawn_future_local(glib::clone!(
+            #[weak(rename_to = obj)]
+            self,
+            async move {
+                obj.change_state_inner(gst::State::Playing).await;
+            }
+        ));
     }
 
     // It is not needed to call this for gst::State::Null.
@@ -613,7 +670,7 @@ impl Viewfinder {
         let (sender, receiver) = futures_channel::oneshot::channel();
 
         let camerabin = self.imp().camerabin();
-        std::thread::spawn(glib::clone!(@weak camerabin => move || {
+        std::thread::spawn(glib::clone!(#[weak] camerabin, move || {
             let timeout = gst::format::ClockTime::from_seconds(2);
             let (res, current_state, pending_state) = camerabin.state(Some(timeout));
             let new_state_is = match res {
@@ -622,12 +679,10 @@ impl Viewfinder {
                         camerabin.set_locked_state(true);
                         log::debug!("Camerabin could not change its state from {current_state:?} to {pending_state:?}");
                         StateChangeState::NotDone
+                    } else if current_state == state {
+                        StateChangeState::Equal
                     } else {
-                        if current_state == state {
-                            StateChangeState::Equal
-                        } else {
-                            StateChangeState::Differ
-                        }
+                        StateChangeState::Differ
                     }
                 }
                 Err(err) => {
@@ -652,8 +707,7 @@ impl Viewfinder {
                 self.set_camerabin_state(state);
             }
             // If the previous state change failed, we might as well try to set it now.
-            StateChangeState::Error => self.set_camerabin_state(state),
-            StateChangeState::Differ => self.set_camerabin_state(state),
+            StateChangeState::Error | StateChangeState::Differ => self.set_camerabin_state(state),
         }
     }
 
@@ -664,9 +718,9 @@ impl Viewfinder {
                 self.imp().set_state(ViewfinderState::Error);
             }
             Ok(gst::StateChangeSuccess::Async) => {
-                log::debug!("Trying to set camerabin state to {state:?}")
+                log::debug!("Trying to set camerabin state to {state:?}");
             }
-            Ok(_) => log::debug!("Camerabin succesfully state set to {state:?}"),
+            Ok(_) => log::debug!("Camerabin successfully state set to {state:?}"),
         }
     }
 
@@ -678,7 +732,7 @@ impl Viewfinder {
             log::error!("Could not pause camerabin: {err}");
             self.imp().set_state(ViewfinderState::Error);
         } else {
-            log::debug!("Camerabin state succesfully set to NULL");
+            log::debug!("Camerabin state successfully set to NULL");
         }
     }
 
@@ -718,19 +772,24 @@ impl Viewfinder {
         self.imp().is_stopping_recording.set(false);
 
         if let Some(path) = self.imp().is_recording_video.take() {
+            self.notify_is_recording();
             let file = gio::File::for_path(path);
             self.emit_recording_done(Some(&file));
         }
     }
 
     fn on_barcode_detected(&self, data_type: crate::CodeType, data: &str) {
-        // We don't emit the signal if we just emited it
+        // We don't emit the signal if we just emitted it
         if self.imp().timeout_handler.borrow().is_none() {
             let id = glib::timeout_add_seconds_local_once(
                 BARCODE_TIMEOUT,
-                glib::clone!(@weak self as obj => move || {
-                    obj.imp().timeout_handler.take();
-                }),
+                glib::clone!(
+                    #[weak(rename_to = obj)]
+                    self,
+                    move || {
+                        obj.imp().timeout_handler.take();
+                    }
+                ),
             );
             self.imp().timeout_handler.replace(Some(id));
             self.emit_code_detected(data_type, data);
@@ -759,6 +818,7 @@ impl Viewfinder {
             self.emit_picture_done(None);
         }
         if imp.is_recording_video.replace(None).is_some() {
+            self.notify_is_recording();
             self.emit_recording_done(None);
         }
         imp.is_stopping_recording.set(false);
@@ -803,27 +863,28 @@ impl Viewfinder {
     fn setup_recording(&self) {
         use gst_pbutils::encoding_profile::EncodingProfileBuilder;
 
+        let caps = gst::Caps::builder("video/webm").build();
+        let mut container_profile = gst_pbutils::EncodingContainerProfile::builder(&caps)
+            .name("WebM audio/video")
+            .description("Standard WebM/VP8/Vorbis");
+
         let video_profile =
             gst_pbutils::EncodingVideoProfile::builder(&gst::Caps::builder("video/x-vp8").build())
                 .preset("Profile Realtime")
                 .variable_framerate(true)
                 .build();
-        let audio_profile = gst_pbutils::EncodingAudioProfile::builder(
-            &gst::Caps::builder("audio/x-vorbis").build(),
-        )
-        .build();
-        let profiles = gst_pbutils::EncodingContainerProfile::builder(
-            &gst::Caps::builder("video/webm").build(),
-        )
-        .name("WebM audio/video")
-        .description("Standard WebM/VP8/Vorbis")
-        .add_profile(video_profile)
-        .add_profile(audio_profile)
-        .build();
+        container_profile = container_profile.add_profile(video_profile);
+
+        if !self.disable_audio_recording() {
+            let audio_profile = gst_pbutils::EncodingAudioProfile::builder(
+                &gst::Caps::builder("audio/x-vorbis").build(),
+            )
+            .build();
+            container_profile = container_profile.add_profile(audio_profile);
+        }
 
         let camerabin = self.imp().camerabin();
-
-        camerabin.set_property("video-profile", profiles);
+        camerabin.set_property("video-profile", container_profile.build());
     }
 
     fn init(&self) {
@@ -842,11 +903,15 @@ impl Viewfinder {
 
         glib::timeout_add_local_once(
             std::time::Duration::from_secs(PROVIDER_TIMEOUT),
-            glib::clone!(@weak self as obj => move || {
-                if matches!(obj.state(), ViewfinderState::Loading) {
-                    obj.imp().set_state(ViewfinderState::NoCameras);
+            glib::clone!(
+                #[weak(rename_to = obj)]
+                self,
+                move || {
+                    if matches!(obj.state(), ViewfinderState::Loading) {
+                        obj.imp().set_state(ViewfinderState::NoCameras);
+                    }
                 }
-            }),
+            ),
         );
     }
 
@@ -860,24 +925,41 @@ impl Viewfinder {
 
         let capsfilter = gst::ElementFactory::make("capsfilter").build()?;
         let decodebin3 = gst::ElementFactory::make("decodebin3").build()?;
+        let identity = gst::ElementFactory::make("identity").build()?;
 
-        let videoflip = gst::ElementFactory::make("videoflip")
-            .property_from_str("video-direction", "auto")
-            .build()?;
-
-        bin.add_many([&device_src, &capsfilter, &decodebin3, &videoflip])?;
-        gst::Element::link_many([&device_src, &capsfilter, &decodebin3])?;
+        bin.add_many([device_src, &capsfilter, &decodebin3, &identity])?;
+        gst::Element::link_many([device_src, &capsfilter, &decodebin3])?;
 
         self.imp().capsfilter.set(capsfilter).unwrap();
 
-        decodebin3.connect_pad_added(glib::clone!(@weak videoflip => move |_, pad| {
+        let (sender, receiver) = futures_channel::oneshot::channel::<bool>();
+        let sender = std::sync::Arc::new(std::sync::Mutex::new(Some(sender)));
+        decodebin3.connect_pad_added(glib::clone!(#[weak] identity, move |_, pad| {
             if pad.stream().is_some_and(|stream| matches!(stream.stream_type(), gst::StreamType::VIDEO)) {
-                pad.link(&videoflip.static_pad("sink").unwrap())
-                   .expect("Failed to link decodebin3:video_%u pad with videoflip:sink");
+                let has_succeeded = pad.link(&identity.static_pad("sink").unwrap())
+                                       .inspect_err(|err| {
+                                           log::error!("Failed to link decodebin3:video_%u pad with identity:sink pad: {err}");
+                                       })
+                                       .is_ok();
+                let mut guard = sender.lock().unwrap();
+                if let Some(sender) = guard.take() {
+                    let _ = sender.send(has_succeeded);
+                }
             }
         }));
 
-        let pad = videoflip.static_pad("src").unwrap();
+        glib::spawn_future_local(glib::clone!(
+            #[weak(rename_to = viewfinder)]
+            self,
+            async move {
+                let has_succeeded = receiver.await.unwrap_or_default();
+                if !has_succeeded {
+                    viewfinder.imp().set_state(ViewfinderState::Error);
+                }
+            }
+        ));
+
+        let pad = identity.static_pad("src").unwrap();
         let ghost_pad = gst::GhostPad::with_target(&pad)?;
         ghost_pad.set_active(true)?;
 
@@ -895,7 +977,7 @@ impl Viewfinder {
         let imp = self.imp();
 
         if let Some(element) = imp.camera_element.get() {
-            camera.reconfigure(&element)?;
+            camera.reconfigure(element)?;
         } else {
             let element = camera.create_element()?;
 
